import os
import datetime

from bs4 import BeautifulSoup
from urllib.parse import unquote

from RAAspotter import RAAspotter


class RAAspotter_pref09(RAAspotter):

    # Config
    __HOST = 'https://www.ariege.gouv.fr'
    __RAA_PAGE = f'{__HOST}/Publications/Recueil-des-actes-administratifs/Recueils-des-Actes-Administratifs-de-l-Ariege-a-partir-du-28-avril-2015'
    __USER_AGENT = 'Mozilla/5.0 (X11; Linux x86_64; rv:109.0) Gecko/20100101 Firefox/115.0'
    full_name = 'Préfecture de l\'Ariège'
    short_code = 'pref09'

    def __init__(self, data_dir):
        super().__init__(data_dir, self.__USER_AGENT)
        self.enable_tor(10)

    def get_raa(self, keywords):
        self.print_output('RAAspotter_pref09')
        self.print_output(f'Termes recherchés: {keywords}')
        self.print_output('')

        pages_to_parse = []

        # Les RAA de l'Ariège sont éparpillés sur des sous-pages par mois.
        # Donc on parse la page principale à la recherche des sous-pages.
        sub_pages = self.get_sub_pages_with_pager(
            self.__RAA_PAGE,
            'div.fr-card__body div.fr-card__content h2.fr-card__title a.fr-card__link',
            'ul.fr-pagination__list li a.fr-pagination__link.fr-pagination__link--next',
            'div.fr-card__body div.fr-card__content div.fr-card__end p.fr-card__detail',
            self.__HOST
        )[::-1]

        # On filtre par date les sous-pages pour limiter les requêtes
        for sub_page in sub_pages:
            guessed_date = datetime.datetime.strptime(sub_page['details'].replace('Publié le ', '').strip(), '%d/%m/%Y')
            guessed_date.replace(day=1)
            if guessed_date >= self.not_before:
                pages_to_parse.append(sub_page['url'])

        # On parse les pages contenant des RAA
        for page in pages_to_parse:
            page_content = self.get_page(page, 'get').content
            raa_elements = self.get_raa_elements(page_content)
            self.parse_raa(raa_elements, keywords.split(','))
        self.mailer()

    def get_raa_elements(self, page_content):
        elements = []
        # On charge le parser
        soup = BeautifulSoup(page_content, 'html.parser')

        # On récupère chaque balise a
        for a in soup.select('div.fr-downloads-group.fr-downloads-group--bordered ul li a'):
            if a.get('href') and a['href'].endswith('.pdf'):
                if a['href'].startswith('/'):
                    url = f"{self.__HOST}{a['href']}"
                else:
                    url = a['href']

                url = unquote(url)
                name = a.find('span').previous_sibling.replace('Télécharger ', '').strip()
                date = datetime.datetime.strptime(a.find('span').get_text().split(' - ')[-1].strip(), '%d/%m/%Y')
                filename = url.split('/')[-1]

                raa = RAAspotter.RAA(url, date, name, filename)
                elements.append(raa)
        return elements
